@@ -163,127 +163,94 @@ def prot(vsini, st_rad):
     
     return prot
     
-def kipping_LD(u1,u2):
+    
+def convert_LD_coeffs(c1, c2, convert_from = "q2u", verify=True):
     """
-    Re-parameterize quadratic limb darkening parameters u1 and u2 according to Kipping (2013) [1]     
+    Convert limb darkening (LD) coefficients (c1,c2) from one LD to another.
     
     Parameters:
-    ----------
-    u1: linear limb darkening coefficient.
-    
-    u2: quadratic limb darkening coefficient.
+    -----------
+    	
+    c1, c2 : float, ufloat, np.array;
+        Limb darkening coefficients to convert.
     
+    convert_from : string;
+        specify which laws to convert from and to.
+        - To convert from u1,u2 to q1,q2 following [1] set convert_from = "u2q".
+        - To convert from q1,q2 to u1,u2 following [1] set convert_from = "q2u".
+        - From h1,h2 to q1,q2 following [1,2] set convert_from = "h2q".
+        - From q1,q2 to h1,h2 following [1,2,3] set convert_from = "q2h".	
+        - From h1,h2 to u1,u2 following [2,3] set convert_from = "h2u".
+        - From u1,u2 to h1,h2 following [2,3] set convert_from = "u2h".
+    Verify: bool;
+    	confirm that the result satisfies the conditions of the LD law as given in Notes from [1,2,3]
+        
     Returns
-    --------
-    q1, q2 : Tuple containing the reparametrized limb darkening coefficients
-    
-    Note:
-    ------
-    Conditions: 0< q1<1, 0<=q2<1
-    
-    References:
-    -----------
-    [1] https://ui.adsabs.harvard.edu/abs/2013MNRAS.435.2152K/abstract
-    
-    """
-    
-    q1 = (u1+u2)**2
-    q2= u1/(2*(u1+u2))
-    
-    return q1, q2
-				
-def kipping_to_quadLD(q1,q2):
-    """
-    Re-parameterize kipping 2013 ldcs q1 and q2 to the usual quadratic limb darkening parameters u1 and u2. according to Kipping (2013) [1] 
-    
-    
-    Parameters:
-    ----------
-    q1: linear limb darkening coefficient.
-    
-    q2: quadratic limb darkening coefficient.
-    
-    Returns:
-    --------
-    u1, u2 : Tuple containing the quadratic limb darkening coefficients
-    
-    Note:
-    ------
-    Conditions: u1+u2<1, u1>0, u1+2u2>0.
-    
-    References:
-    -----------
-    [1] https://ui.adsabs.harvard.edu/abs/2013MNRAS.435.2152K/abstract
-    
-    """
-    u1 = 2 *q1**0.5 *q2 
-    u2 = q1**0.5*(1-2*q2)
-    return u1, u2
-    
-
-def kipping_to_Power2LD(q1, q2):
-    """
-    Re-parameterize kipping (2013)[1] ldcs q1 and q2 to the Power-2 limb darkening parameters h1 and h2 according to (Maxted 2018) [2] and Donald et al 2019 [3].
-    
-    Parameters:
-    ----------
-    q1: linear limb darkening coefficient.
-    
-    q2: quadratic limb darkening coefficient.
-    
-    Returns:
-    --------
-    h1, h2 : Tuple containing the the transformed limb darkening coefficients
+    -------
+    l1, l2 : Tuple;
+    	containing the required reparamterized limb darkening coefficients. if input c1, c2 are numpy arrays then l1 and l2 are arrays. 
     
     Note:
     -------
     Conditions: h1>0,  h2>0,  h2 <= h1, and h1<1.
-    
-    References:
-    ----------
-    [1] https://ui.adsabs.harvard.edu/abs/2013MNRAS.435.2152K/abstract
-    
-    [2] https://ui.adsabs.harvard.edu/abs/2018A%26A...616A..39M/abstract
-    
-    [3] https://ui.adsabs.harvard.edu/abs/2019RNAAS...3..117S/abstract
-    
-    """
-    h1 = 1 - q1**0.5 + q2*q1**0.5 
-    h2 = 1 - q1**0.5
-    return h1, h2
-    
-def Power2_to_kippingLD(h1,h2):
-    """
-    Transform Power-2 limb darkening parameters h1 and h2 (Maxted 2018 [1]) to Kipping (2013 [2]) coefficients. Conditions h1>0,  h2>0,  h2 <= h1, and h1<1. 
-    
-    
-    Parameters:
-    ----------
-    h1: linear limb darkening coefficient.
-    
-    h2: quadratic limb darkening coefficient.
-    
-    Returns
-    --------
-    q1, q2 : Tuple containing the reparametrized limb darkening coefficients
-    
-    Note:
-    ------
-    Conditions: 0< q1<1, 0<=q2<1
+    Conditions: 0< q1<1, 0<=q2<1	
+    Conditions: u1+u2<1, u1>0, u1+2u2>0.
     
     References
     ----------
-    [1] https://ui.adsabs.harvard.edu/abs/2018A%26A...616A..39M/abstract
-    
-    [2] https://ui.adsabs.harvard.edu/abs/2013MNRAS.435.2152K/abstract
-    
+    [1] https://ui.adsabs.harvard.edu/abs/2013MNRAS.435.2152K/abstract    
+    [2] https://ui.adsabs.harvard.edu/abs/2018A%26A...616A..39M/abstract
+    [3] https://ui.adsabs.harvard.edu/abs/2019RNAAS...3..117S/abstract
+           
     """
+    assert convert_from in ['u2q','q2u','h2u','u2h','h2q','q2h'], "invalid input for convert_from. Valid options are 'u2q','q2u','h2u','u2h','h2q','q2h' "
     
-    q1 = (1-h2)**2
-    q2= (h1-h2)/(1-h2)
+    if convert_from == "u2q":
+        q1 = (c1+c2)**2
+        q2 = c1/(2*(c1+c2))
+        if verify: assert np.all(0<q1) and np.all(q1<1) and np.all(0<=q2) and np.all(q2<1), f"obtained values of q1={q1} and q2={q2} do not meet the conditions of [1]. See Notes"         
+        l1, l2 = q1, q2
+        
+    elif convert_from == "q2u":
+        u1 = 2 *c1**0.5 *c2 
+        u2 = c1**0.5*(1-2*c2)
+        if verify: assert np.all((u1+u2)<1) and np.all(u1>0) and np.all((u1+2*u2)>0), f"obtained value of u1={u1} and u2={u2} do not meet the conditions. See Notes" 
+        l1, l2 = u1, u2
+        
+    elif convert_from == "h2q": 
+        q1 = (1-c2)**2
+        q2= (c1-c2)/(1-c2)
+        if verify: assert np.all(0<q1) and np.all(q1<1) and np.all(0<=q2) and np.all(q2<1), f"obtained values of q1={q1} and q2={q2} do not meet the conditions of [1]. See Notes"         
+        l1, l2 = q1, q2
+                   
+    elif convert_from == "q2h": 
+        h1 = 1 - c1**0.5 + c2*c1**0.5 
+        h2 = 1 - c1**0.5
+        if verify: assert np.all(0<h1) and np.all(h1<1) and np.all(0<h2) and np.all(h2<=h1), f"obtained values of h1={h1} and h2={h2} do not meet the conditions of [3]. See Notes"
+        l1, l2 = h1, h2
+        
+    elif convert_from == "h2u":
+        #h2q
+        q1 = (1-c2)**2
+        q2= (c1-c2)/(1-c2)
+        #q2u
+        u1 = 2 *q1**0.5 *q2 
+        u2 = q1**0.5*(1-2*q2) 
+        if verify: assert np.all((u1+u2)<1) and np.all(u1>0) and np.all((u1+2*u2)>0), f"obtained value of u1={u1} and u2={u2} do not meet the conditions. See Notes" 
+        l1, l2 = u1, u2 
+         
+    elif convert_from == "u2h":
+        #u2q
+        q1 = (c1+c2)**2
+        q2 = c1/(2*(c1+c2)) 
+        #q2h
+        h1 = 1 - q1**0.5 + q2*q1**0.5 
+        h2 = 1 - q1**0.5
+        if verify: assert np.all(0<h1) and np.all(h1<1) and np.all(0<h2) and np.all(h2<=h1), f"obtained values of h1={h1} and h2={h2} do not meet the conditions of [3]. See Notes"
+        l1, l2 = h1, h2
+        
+    return l1,l2
     
-    return q1, q2
-				
     
 def aR_to_rho_star(P,aR):
     """
@@ -435,4 +402,39 @@ def timetrans_to_timeperi(tc, per, ecc, omega):
 
     return tp
 
+def r1r2_to_bp(r1,r2,pl=0.01, pu=0.25):
+    """
+    Convert uniform samling of r1 and r2 to impact parameter b and and radius ratio p
+    following Espinoza 2018, https://iopscience.iop.org/article/10.3847/2515-5172/aaef38/meta
+    
+    Paramters:
+    -----------
+    r1, r2: float;
+        uniform parameters in from u(0,1)
+    
+    pl, pu: float;
+        lower and upper limits of the radius ratio
+        
+    
+    Return:
+    -------
+    b, p: tuple;
+        impact paramter and radius ratio
+    """
+    
+    assert 0<r1<=1 and 0<r2<=1, f"r1 and r2 needs to be u(0,1) but r1={r1}, r2={r2}"
+    
+    Ar = (pu-pl)/(2+pu+pl)
+    
+    if r1 > Ar:
+        b = (1+pl) * (1 + (r1-1)/(1-Ar) )
+        p = (1-r2)*pl + r2*pu
+    
+    elif r1 <= Ar:
+        q1 = r1/Ar
+        
+        b = (1+pl) + q1**0.5 * r2*(pu-pl)
+        p = pu + (pl-pu)* q1**0.5*(1-r2)
+    return b, p
+    
     
